import os
import re

from termcolor import colored

from addic7ed.serie_process import pre_process

REGEX = r"(.*)\.[s|S]?([0-9]{1,2})[x|X|e|E]?([0-9]{2})\..*([0-9]+p)?.*"


class FileCrawler:
    def __init__(self):
        self.episodes = {}
        for f in os.listdir():
            if f.endswith((".avi", "mkv", ".mp4")):
                ep = self._parse_filename(f)
                if ep:
                    self.episodes[f] = ep

    def _parse_filename(self, filename):
        m = re.match(REGEX, filename)
        if m:
            return Episode(
                filename,
                pre_process(m.group(1).replace('.', '_').lower()),
                int(m.group(2)),
                int(m.group(3))
            )
        else:
            return None


class Episode:
    def __init__(self, f, serie, season, episode):
        self.infos = {
            "serie": serie,
            "season": season,
            "episode": episode,
        }
        self.filename, self.extension = os.path.splitext(f)
        m = re.search(r"-(.*)%s$" % self.extension, f)
        self.infos["group"] = m.group(1) if m else ""

    def rename(self, new_name):
        try:
            os.rename("%s%s" % (self.filename, self.extension),
                      "%s%s" % (new_name, self.extension))
            ret = colored("Renamed %s to %s" % (self.filename, new_name),
                          "green")
            self.filename = new_name
        except Exception as e:
            ret = colored(e, "red")

        return ret

    def __str__(self):
        return colored("%s - Season %02d Episode %02d (%s)" % (
            self.infos["serie"].replace("_", " ").title(),
            self.infos["season"],
            self.infos["episode"],
            self.filename
        ), "blue")



from termcolor import colored

from .parser import Addic7edParser
from .file_crawler import FileCrawler

crawler = FileCrawler()
parser = Addic7edParser()


def addic7ed():
    try:
        main()
    except (KeyboardInterrupt, SystemExit):
        print(colored("\nBye!", "yellow"))
        exit(0)


def main():
    for filename, ep in crawler.episodes.items():
        subs = parser.parse(**ep.infos)

        print(ep)

        if not subs:
            print(colored("No subtitles for %s" % filename, "red"),
                  end="\n\n")
            continue

        for i, sub in enumerate(subs):
            print("[%s] %s" % (colored(i, "yellow"), sub))

        version = input('Download number? ')

        if not version:
            print(colored("Nothing to do!", "yellow"),
                  end="\n\n")
            continue

        try:
            filename = subs[int(version)].download()
            print(colored("Downloaded %s subtitle file" % filename, "green"))
            if filename:
                print(ep.rename(filename),
                      end="\n\n")
        except Exception as e:
            print(colored(e, "red"),
                  end="\n\n")



from os.path import expanduser
from configparser import ConfigParser

ADDIC7ED_URL = "http://www.addic7ed.com"
# LANG = 52  # Albanian
# LANG = 38  # Arabic
# LANG = 50  # Armenian
# LANG = 48  # Azerbaijani
# LANG = 47  # Bengali
# LANG = 44  # Bosnian
# LANG = 35  # Bulgarian
# LANG = 12  # Català
# LANG = 41  # Chinese (Simplified)
# LANG = 24  # Chinese (Traditional)
# LANG = 31  # Croatian
# LANG = 14  # Czech
# LANG = 30  # Danish
# LANG = 17  # Dutch
LANG = 1   # English
# LANG = 54  # Estonian
# LANG = 13  # Euskera
# LANG = 28  # Finnish
# LANG = 8   # French
# LANG = 53  # French (Canadian)
# LANG = 15  # Galego
# LANG = 11  # German
# LANG = 27  # Greek
# LANG = 23  # Hebrew
# LANG = 55  # Hindi
# LANG = 20  # Hungarian
# LANG = 56  # Icelandic
# LANG = 37  # Indonesian
# LANG = 7   # Italian
# LANG = 32  # Japanese
# LANG = 42  # Korean
# LANG = 57  # Latvian
# LANG = 58  # Lithuanian
# LANG = 49  # Macedonian
# LANG = 40  # Malay
# LANG = 29  # Norwegian
# LANG = 43  # Persian
# LANG = 21  # Polish
# LANG = 9   # Portuguese
# LANG = 10  # Portuguese (Brazilian)
# LANG = 26  # Romanian
# LANG = 19  # Russian
# LANG = 39  # Serbian (Cyrillic)
# LANG = 36  # Serbian (Latin)
# LANG = 60  # Sinhala
# LANG = 25  # Slovak
# LANG = 22  # Slovenian
# LANG = 4   # Spanish
# LANG = 6   # Spanish (Latin America)
# LANG = 5   # Spanish (Spain)
# LANG = 18  # Swedish
# LANG = 59  # Tamil
# LANG = 46  # Thai
# LANG = 16  # Turkish
# LANG = 51  # Ukrainian
# LANG = 45  # Vietnamese

config = ConfigParser()
config.read(expanduser("~") + "/.addic7edrc")
if "addic7ed" in config and "lang" in config["addic7ed"]:
    LANG = config["addic7ed"]["lang"]



def pre_process(serie):
    if serie.startswith("marvels_daredevil"):
        return serie.replace("marvels_", "")
    elif serie.startswith("marvels_"):
        return serie.replace("marvels", "marvel's")
    elif serie == "the_flash_2014":
        return "the_flash_(2014)"
    elif serie == "mr_robot":
        return "mr._robot"
    elif serie == "house_of_cards_2013":
        return "house_of_cards_(2013)"
    elif serie == "doctor_who_2005":
        return "doctor_who"
    elif serie == "ash_vs_evil_dead":
        return "ash_vs._evil_dead"
    else:
        return serie


def post_process(serie):
    if serie.startswith("Daredevil -"):
        return serie.replace("Daredevil -", "Marvel's Daredevil -")
    elif serie.startswith("Doctor Who -"):
        return serie.replace("Doctor Who -", "Doctor Who (2005) -")
    else:
        return serie
